@@ -43,6 +43,7 @@ private:
     void root_value();
     void value();
     void array();
+    void end_array();
     void object();
     void number();
     void string();
@@ -61,9 +62,11 @@ void json_parser<_Handler>::parse()
 {
     m_handler.begin_parse();
 
-    skip_blanks();
+    skip_ws();
     if (has_char())
         root_value();
+    else
+        throw json::parse_error("parse: no json content could be found in file", offset());
 
     if (has_char())
         throw json::parse_error("parse: unexpected trailing string segment.", offset());
@@ -139,28 +142,30 @@ void json_parser<_Handler>::array()
     m_handler.begin_array();
     for (next(); has_char(); next())
     {
+        skip_ws();
+
         if (cur_char() == ']')
         {
-            m_handler.end_array();
-            next();
-            skip_blanks();
+            end_array();
             return;
         }
 
-        skip_blanks();
         value();
-        skip_blanks();
+        skip_ws();
 
         if (has_char())
         {
             switch (cur_char())
             {
                 case ']':
-                    m_handler.end_array();
-                    next();
-                    skip_blanks();
+                    end_array();
                     return;
                 case ',':
+                    if (next_char() == ']')
+                    {
+                        json::parse_error::throw_with(
+                            "array: ']' expected but '", cur_char(), "' found.", offset() );
+                    }
                     continue;
                 default:
                     json::parse_error::throw_with(
@@ -179,24 +184,38 @@ void json_parser<_Handler>::array()
     throw json::parse_error("array: failed to parse array.", offset());
 }
 
+template<typename _Handler>
+void json_parser<_Handler>::end_array()
+{
+    m_handler.end_array();
+    next();
+    skip_ws();
+}
+
 template<typename _Handler>
 void json_parser<_Handler>::object()
 {
     assert(cur_char() == '{');
 
+    bool require_new_key = false;
     m_handler.begin_object();
     for (next(); has_char(); next())
     {
-        skip_blanks();
+        skip_ws();
         if (!has_char())
             throw json::parse_error("object: stream ended prematurely before reaching a key.", offset());
 
         switch (cur_char())
         {
             case '}':
+                if (require_new_key)
+                {
+                    json::parse_error::throw_with(
+                        "object: new key expected, but '", cur_char(), "' found.", offset());
+                }
                 m_handler.end_object();
                 next();
-                skip_blanks();
+                skip_ws();
                 return;
             case '"':
                 break;
@@ -204,6 +223,7 @@ void json_parser<_Handler>::object()
                 json::parse_error::throw_with(
                     "object: '\"' was expected, but '", cur_char(), "' found.", offset());
         }
+        require_new_key = false;
 
         parse_quoted_string_state res = parse_string();
         if (!res.str)
@@ -220,35 +240,36 @@ void json_parser<_Handler>::object()
 
         m_handler.object_key(res.str, res.length, res.transient);
 
-        skip_blanks();
+        skip_ws();
         if (cur_char() != ':')
             json::parse_error::throw_with(
                 "object: ':' was expected, but '", cur_char(), "' found.", offset());
 
         next();
-        skip_blanks();
+        skip_ws();
 
         if (!has_char())
             throw json::parse_error("object: stream ended prematurely before reaching a value.", offset());
 
         value();
 
-        skip_blanks();
+        skip_ws();
         if (!has_char())
-            throw json::parse_error("object: stream ended prematurely before reaching either ']' or ','.", offset());
+            throw json::parse_error("object: stream ended prematurely before reaching either '}' or ','.", offset());
 
         switch (cur_char())
         {
             case '}':
                 m_handler.end_object();
                 next();
-                skip_blanks();
+                skip_ws();
                 return;
             case ',':
+                require_new_key = true;
                 continue;
             default:
                 json::parse_error::throw_with(
-                    "object: either ']' or ',' expected, but '", cur_char(), "' found.", offset());
+                    "object: either '}' or ',' expected, but '", cur_char(), "' found.", offset());
         }
     }
 
@@ -262,7 +283,7 @@ void json_parser<_Handler>::number()
 
     double val = parse_double_or_throw();
     m_handler.number(val);
-    skip_blanks();
+    skip_ws();
 }
 
 template<typename _Handler>
